(* ::Package:: *)

(* AutoDipole 1.2 - Package of an automated dipole subtraction by
Kouhei Hasegawa, Sven Moch, and Peter Uwer *)


(* Code to convert to FORTRAN forms *)


Dipolefortran:=Module[{},
settargetdipolefortran;
type=FileType[targetdir];
If[ type==Directory,
DeleteDirectory[targetdir,DeleteContents -> True];
];
CreateDirectory[targetdir];
CreateDirectory[targettempdir];
namedipolef=StringJoin[targetdir,"/dipole.f"];
namereducedm=StringJoin[targetdir,"/reducedm.f"];
str=OpenWrite[namedipolef,FormatType->FortranForm,PageWidth->65];
str2=OpenWrite[namereducedm,FormatType->FortranForm,PageWidth->65];
createfortran;Close[str];Close[str2];]


settargetdipolefortran:=Module[{},
BL[1]=A[1];BL[2]=A[2];
Do[BL[k+2]=Part[flistm,k],{k,1,nfinal}];
Do[
If[BL[k]==u || BL[k]==d || BL[k]==b || BL[k]==t || BL[k]==g,ML[k]=ToString[BL[k]]];
If[BL[k]==ubar, ML[k]="ux"];
If[BL[k]==dbar, ML[k]="dx"];
If[BL[k]==bbar, ML[k]="bx"];
If[BL[k]==tbar, ML[k]="tx"];
If[BL[k]==e,ML[k]="e-"];
If[BL[k]==ebar, ML[k]="e+"];
If[BL[k]==muon,ML[k]="mu-"];
If[BL[k]==muonbar, ML[k]="mu+"];
If[BL[k]==gamma, ML[k]="a"];
If[BL[k]==Wp, ML[k]="w+"];
If[BL[k]==Wm, ML[k]="w-"];
If[BL[k]==Z, ML[k]="z"];
If[BL[k]==h, ML[k]="h"];
,{k,1,nfinal+2}];
processdir=StringJoin["Proc_",ML[1],ML[2],"_"];
Do[processdir=StringJoin[processdir,ML[k]],{k,3,nfinal+2}];
targetdir=StringJoin[packagedir,"/process/",processdir];
targettempdir=StringJoin[packagedir,"/process/",processdir,"/temp"];

];


(* Create fortran file *)


createfortran:=Module[{},
                  beginformat;
                  writescalar;
                  callsubroutine;
                  WriteString[str,"c Here is start of dipole creation \n","\n"];
                  writeehelicity;
                  writedipole;
                  WriteString[str,"c Here is end of dipole creation \n","\n"];
                  writeSumD;
                  endformat[str];
                  writefundot;
                  writefunph;
                  writefunsp;
                  writefunspc;
                  writefunspz;
                  writefunspzc;
                  writefunzspz;
                  writesubreducedm;
]


(* write subroutine and variable names on dipole.f *)


beginformat:=Module[{},
WriteString[str,
"c Generated by AutoDipole Version 1.2, \n",
"c Kouhei Hasegawa, Sven Moch, and Peter Uwer, Nucl.Phys.Proc.Suppl.183:268-273,2008  \n",
"c \n"];
WriteString[str,"c NLO process: ",{{A[1],pa},{A[2],pb}}," --> ", tfinal," \n"];
WriteString[str,"      subroutine dipole(p,dip,SumD) \n"];
WriteString[str,"      implicit none \n","\n"];
WriteString[str,"      integer i,j \n",
"      double precision p(0:3,1:",nfinal+2,"),dip(",dtotal,"),SumD \n",
"      double precision Pi,rtwo,AL,CF,CA,TR,mt,mb \n",
"      double complex eh(",dtotal,") \n",
"      double precision ptil(0:3,1:",nborn,",",co,") \n",
"      double precision pem(0:3,1:",co,"),plk(0:3,1:",co,")  \n",
"      double precision dot,zspz \n",
"      double complex ph,sp,spc,spz,spzc \n"
];
If[dAs != 0,
WriteString[str,"      double precision B1(",dAs,") \n"];
WriteString[str,"      double complex B1dh(",dAs,") \n"]
];
If[dBs != 0,
WriteString[str,"      double precision B2(",dAs+1,":",dAs+dBs,") \n"];
WriteString[str,"      double complex B2dh(",dAs+1,":",dAs+dBs,") \n"]
];
If[dCs != 0,
WriteString[str,"      double precision B3(",dAs+dBs+1,":",dAs+dBs+dCs,") \n"];
WriteString[str,"      double complex B3dh(",dAs+dBs+1,":",dAs+dBs+dCs,") \n"]
];
If[dDs != 0,
WriteString[str,"      double precision B4(",dAs+dBs+dCs+1,":",dAs+dBs+dCs+dDs,") \n"]
];
WriteString[str,"      double precision "];
WriteString[str,"sab,"];
Do[
WriteString[str,"sa",i,","]; 
WriteString[str,"sb",i,","];
 ,{i,1,nfinal}];
WriteString[str," \n"];
WriteString[str,"     & "];
Do[ 
Do[
If[j > i, 
If[i != nfinal-1 || j != nfinal,WriteString[str,"s",i,j,","]];
If[i == nfinal-1 && j == nfinal,WriteString[str,"s",i,j]];
]
 ,{j,1,nfinal}];
,{i,1,nfinal}];
WriteString[str," \n \n"];
WriteString[str,
"      parameter(rtwo=1.4142135623730951d0)\n",
"      parameter(Pi=3.141592653589793238D0)\n",
"      parameter(CF=4.D0/3.D0,CA=3.D0,TR=0.5D0)\n"
];
WriteString[str,"      common /MASS/ mt,mb  \n"];
WriteString[str,"      common /SCALAR/"];
WriteString[str,"sab,"];
Do[
WriteString[str,"sa",i,","];
WriteString[str,"sb",i,","];
,{i,1,nfinal}];
WriteString[str," \n"];
WriteString[str,"     & "];
Do[
Do[
If[j > i, 
If[i != nfinal-1 || j != nfinal,WriteString[str,"s",i,j,","]];
If[i == nfinal-1 && j == nfinal,WriteString[str,"s",i,j]];
]
,{j,i+1,nfinal}];
,{i,1,nfinal}];

WriteString[str,"    \n"];
WriteString[str,"      common /OUTPUTM/ ptil  \n"];
WriteString[str,"      common /OTHERM/ pem,plk  \n"];
WriteString[str,"      common /usedalpha/ AL  \n"];
WriteString[str,"    \n \n"];
];


writescalar:=Module[{},
WriteString[str,"  \n"];
Do[ 
Do[
  If[j != i, 
    WriteString[str,"      s",i,j,"=2.d0*dot(p(0,",i+2,"),p(0,",j+2,")) \n"];
  ]
,{j,i,nfinal}];
,{i,1,nfinal}];
Do[ 
    WriteString[str,"      sa",i,"=2.d0*dot(p(0,",1,"),p(0,",i+2,"))  \n"];
 ,{i,1,nfinal}];
Do[ 
    WriteString[str,"      sb",i,"=2.d0*dot(p(0,",2,"),p(0,",i+2,"))  \n"];
 ,{i,1,nfinal}];
    WriteString[str,"      sab=2.d0*dot(p(0,1),p(0,2))  \n"];
WriteString[str,"  \n"];
]


callsubroutine:=Module[{},
WriteString[str,
"      call reducedm(p) \n",
"      \n"];

If[dAs !=0,
 WriteString[str,
 "      do i=1,",dAs," \n",
 "        call cmatrix1(ptil(0,1,",i,"),",i,",B1(",i,")) \n",
 "        B1(",i,")=B1(",i,")/",ng,".d0 \n",
 "      enddo \n",
 "      \n"];
 Do[
  If[gluonemit[k,1]==1,
     WriteString[str,
     "      call cmatrix1dh(ptil(0,1,",k,"),",k,",B1dh(",k,"),-1,1) \n",
     "      B1dh(",k,")=B1dh(",k,")/",ng,".d0 \n"
     ]];
  If[gluonemit[k,1]==2,
     WriteString[str,
     "      call cmatrix1dh(ptil(0,1,",k,"),",k,",B1dh(",k,"),1,-1) \n",
     "      B1dh(",k,")=B1dh(",k,")/",ng,".d0 \n"
     ]];
  ,{k,1,dtotal}];
WriteString[str," \n"];
];

If[dBs !=0,

 Do[
  If[speciesdip2[k]==1,
     WriteString[str,
     "      call cmatrix2u(ptil(0,1,",k,"),",k,",B2(",k,")) \n",
     "      B2(",k,")=B2(",k,")*",ng+1,".d0 \n"
   ]];
  If[speciesdip2[k]==2,
     WriteString[str,
     "      call cmatrix2d(ptil(0,1,",k,"),",k,",B2(",k,")) \n",
     "      B2(",k,")=B2(",k,")*",ng+1,".d0 \n"
   ]];
  If[speciesdip2[k]==3,
     WriteString[str,
     "      call cmatrix2b(ptil(0,1,",k,"),",k,",B2(",k,")) \n",
     "      B2(",k,")=B2(",k,")*",ng+1,".d0 \n"
   ]];
  If[speciesdip2[k]==4,
     WriteString[str,
     "      call cmatrix2t(ptil(0,1,",k,"),",k,",B2(",k,")) \n",
     "      B2(",k,")=B2(",k,")*",ng+1,".d0 \n"
   ]];
,{k,dAs+1,dAs+dBs}];
WriteString[str," \n"];

 Do[
  If[speciesdip2[k]==1,
     WriteString[str,
     "      call cmatrix2udh(ptil(0,1,",k,"),",k,",B2dh(",k,"),-1,1) \n",
     "      B2dh(",k,")=B2dh(",k,")*",ng+1,".d0 \n"
   ]];
  If[speciesdip2[k]==2,
     WriteString[str,
     "      call cmatrix2ddh(ptil(0,1,",k,"),",k,",B2dh(",k,"),-1,1) \n",
     "      B2dh(",k,")=B2dh(",k,")*",ng+1,".d0 \n"
   ]];
  If[speciesdip2[k]==3,
     WriteString[str,
     "      call cmatrix2bdh(ptil(0,1,",k,"),",k,",B2dh(",k,"),-1,1) \n",
     "      B2dh(",k,")=B2dh(",k,")*",ng+1,".d0 \n"
   ]];
  If[speciesdip2[k]==4,
     WriteString[str,
     "      call cmatrix2tdh(ptil(0,1,",k,"),",k,",B2dh(",k,"),-1,1) \n",
     "      B2dh(",k,")=B2dh(",k,")*",ng+1,".d0 \n"
   ]];
,{k,dAs+1,dAs+dBs}];
WriteString[str," \n"];
];

If[dCs !=0,
 Do[
  If[gluonemit[k,3]==21,
     WriteString[str,
     "      call cmatrix3u(ptil(0,1,",k,"),",k,",B3(",k,")) \n"
   ]];
  If[gluonemit[k,3]==22,
     WriteString[str,
     "      call cmatrix3ubar(ptil(0,1,",k,"),",k,",B3(",k,")) \n"
   ]];
  If[gluonemit[k,3]==23,
     WriteString[str,
     "      call cmatrix3d(ptil(0,1,",k,"),",k,",B3(",k,")) \n"
   ]];
  If[gluonemit[k,3]==24,
     WriteString[str,
     "      call cmatrix3dbardh(ptil(0,1,",k,"),",k,",B3(",k,")) \n"
   ]];
,{k,1,dtotal}];

WriteString[str," \n"];

 Do[
  If[gluonemit[k,3]==21,
     WriteString[str,
     "      call cmatrix3udh(ptil(0,1,",k,"),",k,",B3dh(",k,"),1,-1) \n"
   ]];
  If[gluonemit[k,3]==22,
     WriteString[str,
     "      call cmatrix3ubardh(ptil(0,1,",k,"),",k,",B3dh(",k,"),1,-1) \n"
   ]];
  If[gluonemit[k,3]==23,
     WriteString[str,
     "      call cmatrix3ddh(ptil(0,1,",k,"),",k,",B3dh(",k,"),1,-1) \n"
   ]];
  If[gluonemit[k,3]==24,
     WriteString[str,
     "      call cmatrix3dbardh(ptil(0,1,",k,"),",k,",B3dh(",k,"),1,-1) \n"
   ]];
,{k,1,dtotal}];
WriteString[str," \n"];
];
If[dDs !=0,

 Do[
  If[speciesdip4[k]==1,
     WriteString[str,
     "      call cmatrix4u(ptil(0,1,",k,"),",k,",B4(",k,")) \n"
   ]];
  If[speciesdip4[k]==2,
     WriteString[str,
     "      call cmatrix4ubar(ptil(0,1,",k,"),",k,",B4(",k,")) \n"
   ]];
  If[speciesdip4[k]==3,
     WriteString[str,
     "      call cmatrix4d(ptil(0,1,",k,"),",k,",B4(",k,")) \n"
   ]];
  If[speciesdip4[k]==4,
     WriteString[str,
     "      call cmatrix4dbardh(ptil(0,1,",k,"),",k,",B4(",k,")) \n"
   ]];
,{k,dAs+dBs+dCs+1,dAs+dBs+dCs+dDs}];
WriteString[str," \n"];

];

WriteString[str," \n"];
]


writeehelicity:=Module[{},  
Do[
  If[gluonemit[i,1]==1 || gluonemit[i,1]==2 || gluonemit[i,2]==1 
     || gluonemit[i,3]==21 || gluonemit[i,3]==22 || gluonemit[i,3]==23 || gluonemit[i,3]==24,
   WriteString[str,"      eh(",i,")=\n"];
   WriteString[str,"     & "];
   Write[str,ehS[i]];
   WriteString[str,"   \n"];
  ]
,{i,1,dtotal}];
WriteString[str,"   \n"];
]


writedipole:=Module[{},  
Do[
  WriteString[str,"      dip(",i,")=\n"];
  WriteString[str,"     & "];
  Write[str,dipS[i]];
  WriteString[str,"   \n"];
  ,{i,1,dtotal}];
  WriteString[str,"   \n"];
]


writeSumD:=Module[{},
WriteString[str,"      SumD = 0.d0 \n",
"      do i=1,",dtotal," \n",
"      SumD = SumD + dip(i) \n",
"      enddo",
"\n","\n"];
]


endformat[str_]:=Module[{},WriteString[str,"      return \n","      end \n","\n"];]


(* Write subroutine and function *)


writefundot:=Module[{},
WriteString[str,"\n \n      double precision function dot(p1,p2) \n",
"      implicit none \n",
"      double precision p1(0:3),p2(0:3) \n",
"      dot=p1(0)*p2(0)-p1(1)*p2(1)-p1(2)*p2(2)-p1(3)*p2(3) \n",
"      end \n"];]


writefunph:=Module[{},
WriteString[str,"\n \n
      double complex function ph(k) \n",
"      implicit none \n",
"      double precision k(0:3) \n",
"      ph=dcmplx(k(1),k(2))/sqrt(k(1)**2+k(2)**2)  \n",
"      end \n"]
]


writefunsp:=Module[{},
WriteString[str,"\n \n
       double complex function sp(k1,k2) \n",
"      implicit none \n",
" \n",
"      double precision k1(0:3),k2(0:3),k1m,k1p,k2m,k2p \n",
"      double complex ph \n",
" \n",
"      k1m=k1(0)-k1(3) \n",
"      k1p=k1(0)+k1(3) \n",
"      k2m=k2(0)-k2(3) \n",
"      k2p=k2(0)+k2(3) \n",
"      sp=sqrt(k1m*k2p)*ph(k1)-sqrt(k1p*k2m)*ph(k2) \n",
"      end \n"]
]


writefunspc:=Module[{},
WriteString[str,"\n \n
       double complex function spc(k1,k2) \n",
"      implicit none \n",
" \n",
"      double precision k1(0:3),k2(0:3) \n",
"      double complex sp \n",
" \n",
"      spc=dconjg(sp(k1(0),k2(0))) \n",
"      end \n"]
]


writefunspz:=Module[{},
WriteString[str,"\n \n
       double complex function spz(k,l) \n",
"      implicit none  \n",
"      double precision k(0:3),l(0:3),km,kp \n",
"      double complex ph  \n",
"      km=k(0)-k(3)  \n",
"      kp=k(0)+k(3)  \n",
"      if (l(3) .gt. 0.d0) then \n",
"         spz= sqrt(2.d0*l(0)*km)*ph(k) \n",
"      elseif (l(3) .lt. 0.d0) then \n",
"         spz= -sqrt(2.d0*l(0)*kp) \n",
"      endif \n",
"      end  \n"]
]


writefunspzc:=Module[{},
WriteString[str,"\n \n
       double complex function spzc(k,l) \n",
"      implicit none \n",
" \n",
"      double precision k(0:3),l(0:3) \n",
"      double complex spz \n",
" \n",
"      spzc=dconjg(spz(k(0),l(0))) \n",
"      end \n"]
]


writefunzspz:=Module[{},
WriteString[str,"\n \n
      double precision function zspz(l1,l2) \n",
"      implicit none  \n",
"      double precision l1(0:3),l2(0:3) \n",
"      if (l1(3) .gt. 0.d0 .and. l2(3) .lt. 0.d0) then \n",
"         zspz= -2.d0*sqrt(l1(0)*l2(0)) \n",
"      elseif (l1(3) .lt. 0.d0 .and. l2(3) .gt. 0.d0) then \n",
"         zspz= 2.d0*sqrt(l1(0)*l2(0)) \n",
"      endif \n",      
"      end  \n"]
]


writesubreducedm:=Module[{},writereducedmhead;writereducedmbody;writereducedmmap;]


writereducedmhead:=Module[{},
WriteString[str2,
"c Generated by AutoDipole Version 1.2, \n",
"c Kouhei Hasegawa, Sven Moch, and Peter Uwer, Nucl.Phys.Proc.Suppl.183:268-273,2008  \n",
"c \n"];
WriteString[str2,"c NLO process: ",{{A[1],pa},{A[2],pb}}," --> ", tfinal," \n"];
WriteString[str2,"      subroutine reducedm(p) \n",
"      implicit none \n","\n",
"      integer r \n",
"      double precision p(0:3,1:",nfinal+2,"),ptil(0:3,1:",nborn,",",co,") \n",
"      double precision pem(0:3,1:",co,"),psp(0:3,1:",co,"),ktil(0:3,1:",nfinal+2,",1:",co,")  \n",
"      double precision plk(0:3,1:",co,"),mt,mb \n"];
WriteString[str2,"      double precision "];
WriteString[str2,"sab,"];
Do[
WriteString[str2,"sa",i,","]; 
WriteString[str2,"sb",i,","];
 ,{i,1,nfinal}];
WriteString[str2," \n"];
WriteString[str2,"     & "];
Do[ 
Do[
If[j > i, 
If[i != nfinal-1 || j != nfinal,WriteString[str2,"s",i,j,","]];
If[i == nfinal-1 && j == nfinal,WriteString[str2,"s",i,j]];
]
 ,{j,1,nfinal}];
,{i,1,nfinal}];
WriteString[str2," \n \n"];

WriteString[str2," \n",
"      common /MASS/ mt,mb  \n"];

WriteString[str2,"      common /SCALAR/"];
WriteString[str2,"sab,"];
Do[
WriteString[str2,"sa",i,","];
WriteString[str2,"sb",i,","];
,{i,1,nfinal}];
WriteString[str2," \n"];
WriteString[str2,"     & "];
Do[
Do[
If[j > i, 
If[i != nfinal-1 || j != nfinal,WriteString[str2,"s",i,j,","]];
If[i == nfinal-1 && j == nfinal,WriteString[str2,"s",i,j]];
]
,{j,i+1,nfinal}];
,{i,1,nfinal}];
WriteString[str2," \n"];
WriteString[str2,"      common /OUTPUTM/ ptil  \n"];
WriteString[str2,"      common /OTHERM/ pem,plk  \n"];
    WriteString[str2,"    \n \n"];
    WriteString[str2,"      do r = 0,3 \n \n"];]


writereducedmbody:=Module[{},
Do[
   WriteString[str2,"      pem(r,",i,")=\n"];
   WriteString[str2,"     & "];
   Write[str2,pemS[i]];
   WriteString[str2,"   \n"];
 ,{i,1,dtotal}];
  WriteString[str2,"   \n"];

Do[
  If[dinin[i]==0,
    WriteString[str2,"      psp(r,",i,")=\n"];
    WriteString[str2,"     & "];
    Write[str2,pspS[i]];
    WriteString[str2,"   \n"];]
 ,{i,1,dtotal}];
  WriteString[str2,"   \n"];

Do[
  If[dinin[i]==1,
   Do[
    WriteString[str2,"      ktil(r,",cor[i,j]+2,",",i,")=\n"];
    WriteString[str2,"     & "];
    Write[str2,ktilS[cor[i,j],i]];
    ,{j,3,nborn}];
    WriteString[str2,"   \n"];]
 ,{i,1,dtotal}];
  WriteString[str2,"   \n"];

Do[
  If[less[i]==1,
    WriteString[str2,"      plk(r,",i,")=\n"];
    WriteString[str2,"     & "];
    Write[str2,plesskS[i]];
    WriteString[str2,"   \n"];]
 ,{i,1,dtotal}];
  WriteString[str2,"   \n"];
]


writereducedmmap:=Module[{},
Do[
 If[dinin[i] == 0,
  Do[
   If[j == emit[i], 
     WriteString[str2,"      ptil(r,",j,",",i,")=pem(r,",i,") \n"];
     ];
   If[j == spec[i], 
     WriteString[str2,"      ptil(r,",j,",",i,")=psp(r,",i,") \n"];
     ];
   If[cor[i,j]==a, 
     WriteString[str2,"      ptil(r,",j,",",i,")=p(r,1) \n"];
     ];
   If[cor[i,j]==b, 
     WriteString[str2,"      ptil(r,",j,",",i,")=p(r,2) \n"];
     ];
   If[IntegerQ[cor[i,j]] && cor[i,j] != 0, 
     WriteString[str2,"      ptil(r,",j,",",i,")=p(r,",cor[i,j]+2,") \n"];
     ];
  ,{j,1,nborn}];
  ];

 If[dinin[i] == 1,
  Do[
   If[j == emit[i], 
     WriteString[str2,"      ptil(r,",j,",",i,")=pem(r,",i,") \n"];
     ];
   If[j == spec[i] && cor[i,j]==a, 
     WriteString[str2,"      ptil(r,",j,",",i,")=p(r,1) \n"];
     ];
   If[j == spec[i] && cor[i,j]==b, 
     WriteString[str2,"      ptil(r,",j,",",i,")=p(r,2) \n"];
     ];

(*
   If[cor[i,j]==a, 
     WriteString[str2,"      ptil(r,",j,",",i,")=ktil(r,1,",i,") \n"];
     ];
   If[cor[i,j]==b, 
     WriteString[str2,"      ptil(r,",j,",",i,")=ktil(r,2,",i,") \n"];
     ]; 
*)
   If[IntegerQ[cor[i,j]] && cor[i,j] != 0, 
     WriteString[str2,"      ptil(r,",j,",",i,")=ktil(r,",cor[i,j]+2,",",i,") \n"];
     ];
  ,{j,1,nborn}];
  ];
     WriteString[str2,"    \n"];
,{i,1,co}];

     WriteString[str2,"    \n"];
    WriteString[str2,"      enddo "];

     WriteString[str2,"    \n \n"];
    WriteString[str2,"      return \n"];
    WriteString[str2,"      end "];
] 
